<?php

namespace Intergo\RedisProxy;

use Illuminate\Cache\CacheManager;
use Illuminate\Redis\RedisManager;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\ServiceProvider;
use Intergo\RedisProxy\Proxies\CacheProxy;
use Intergo\RedisProxy\Proxies\RedisProxy;

class RedisProxyServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        if (function_exists('config_path')) { // function not available and 'publish' not relevant in Lumen
            $this->publishes([__DIR__ . '/../config/redis-proxy.php' => config_path('redis-proxy.php')], 'config');
        }
    }

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(__DIR__ . '/../config/redis-proxy.php', 'redis-proxy');

        foreach(config('redis-proxy.cache.stores') as $name => $parameters){
            if($name == "redis") $name = "";
            $this->app->singleton(CacheProxy::class.$name, function($app) use($parameters,$name){
                $cache = new CacheManager($app);
                return new CacheProxy($cache, $parameters);
            });
        }

        foreach(config('redis-proxy.redis.connections') as $name => $parameters) {
            if($name == "default") $name = "";
            $this->app->singleton(
                RedisProxy::class.$name,
                function ($app) use($parameters,$name){
                    $redisconfig = config('database.redis');
                    $redisClient = $redisconfig['client'] ?? 'predis';
                    $redis = new RedisManager($app, $redisClient, $redisconfig);
                    return new RedisProxy($redis, $parameters);
                }
            );
        }
    }
}
